<?php

require '../vendor/autoload.php';

//JSON validator
use JsonSchema\Validator;

//URL query string handler
use League\Uri\Parser\QueryString;

//Database lib
use Medoo\Medoo;

//Pretty errors in JSON
$whoops = new \Whoops\Run;
$whoops->pushHandler(new \Whoops\Handler\JsonResponseHandler);
$whoops->register();

//Guzzle http
$client = new GuzzleHttp\Client();

$validator = new Validator();

$config = json_decode(file_get_contents("../config.json"));

$validator->validate($config, (object) ['$ref' => 'file://' . realpath('../configSchema.json')]);

if (!$validator->isValid()) {
    // foreach ($validator->getErrors() as $error) {
    //     echo sprintf("[%s] %s\n", $error['property'], $error['message']);
    // }
    $error = $validator->getErrors()[0]; //just show one error at a time
    throw new Exception(sprintf("[%s] %s\n", $error['property'], $error['message']));
}

//Convert key,value pairs into an object for easier referencing
$parsedQuery = [];
foreach (QueryString::parse($_SERVER['QUERY_STRING']) as $q) {
    $parsedQuery[$q[0]] = $q[1];
}

//Check auth key
if (!array_key_exists('key', $parsedQuery) || $parsedQuery['key'] !== $config->key) {
    throw new Exception('Invalid Key');
}

//Set up databases
$databases = [];
foreach ($config->connections as $connection) {
    array_push($databases, (object) [
        "connection" => $connection,
        "database" => new Medoo((array) $connection->database),
    ]);
}

$results = [];

//Get data from databases and fields
foreach ($databases as $database) {
    foreach ((object) $database->connection->fields as $field) {
        switch ($field->info) {
            case "SUM":
                $value = $database->database->sum($field->table, $field->column);
                break;
            case "MIN":
                $value = $database->database->min($field->table, $field->column);
                break;
            case "MAX":
                $value = $database->database->max($field->table, $field->column);
                break;
            case "COUNT":
                $value = $database->database->count($field->table);
                break;
            case "AVG":
                $value = $database->database->avg($field->table, $field->column);
                break;
            case "LAST":
                $value = $database->database->get(
                    $field->table,
                    [
                        $field->column,
                    ],
                    [
                        "ORDER" => [
                            // Order by column with descending sorting
                            $field->column => "DESC",
                        ],
                        'LIMIT' => 1,
                    ]
                )[$field->column];
                break;
            default:
                //Nothing
        }

        array_push($results, (object) [
            "description" => $field->description,
            "value" => $value,
        ]);
    }
}

//Get JSON from relays
foreach ($config->relays as $relay) {
    $res = json_decode($client->request('GET', $relay->url)->getBody());
    array_push($results, (object) ["description" => $relay->description, "value" => $res]);
}

//Get Disk Info
function formatBytes($bytes, $precision = 2)
{
    $units = array('B', 'KB', 'MB', 'GB', 'TB');

    $bytes = max($bytes, 0);
    $pow = floor(($bytes ? log($bytes) : 0) / log(1024));
    $pow = min($pow, count($units) - 1);

    // Uncomment one of the following alternatives
    $bytes /= pow(1024, $pow);
    // $bytes /= (1 << (10 * $pow));

    return round($bytes, $precision) . ' ' . $units[$pow];
}

foreach ($config->disk_info as $disk) {
    $freeBytes = disk_free_space($disk->path);
    $totalBytes = disk_total_space($disk->path);
    $disk_info = [
        "free_space" => formatBytes($freeBytes),
        "total_space" => formatBytes($totalBytes),
    ];
    array_push($results, (object) ["description" => $disk->path . ' ' . $disk->name, "value" => $disk_info]);
}

//Get System Info - some of these commands are linux specific
function getSystemMemInfo()
{
    $data = explode("\n", file_get_contents("/proc/meminfo"));
    $memInfo = array();
    foreach ($data as $line) {
        //Only explode if the line contains ":"
        if (strpos($line, ":")) {
            list($key, $val) = explode(":", $line);
        }
        $val = trim($val);
        $key = trim($key);
        $value = (object) [$key => $val];
        if (in_array($key, ["MemTotal", "MemFree", "MemAvailable", "SwapTotal", "SwapFree", "Cached", "SwapCached"])) {
            array_push($memInfo, $value);
        }

    }
    return $memInfo;
}

if ($config->system_info->top) {
    if (PHP_OS == "Linux") {
        //The top command doesn't play nice outputting text to PHP, so ps is used instead to show the top 5 CPU processes
        array_push($results, (object) ["description" => "Top Processes", "value" => shell_exec("ps -Ao user,comm,pcpu --sort=-pcpu | head -n 6")]);
    }
}
if ($config->system_info->meminfo) {
    if (PHP_OS == "Linux") {
        array_push($results, (object) ["description" => "Memory Info", "value" => getSystemMemInfo()]);
    }
}
if ($config->system_info->uptime) {
    array_push($results, (object) ["description" => "Uptime", "value" => shell_exec('uptime')]);
}

header('Content-Type: application/json');
echo json_encode($results, JSON_PRETTY_PRINT);
